const mdpAudierElementor = {
    audierElementor: function ( widgetId ) {
        const $box = document.querySelector( `.elementor-element-${widgetId} .mdp-audier-elementor-box` );
        const features = JSON.parse( $box.dataset.controls );
        const playerBoolSettings = JSON.parse( $box.dataset.playerBoolSettings );
        const isPlaylist = $box.dataset.source === 'playlist';
        const playerControlsSettings = JSON.parse( $box.dataset.playerSettings );

        // set player settings
        const playerSettings = {
            features: features,
        };

        console.log( features );

        // set playlist controls settings
        for ( let playlistSetting in playerControlsSettings ) {
            playerSettings[playlistSetting] = playerControlsSettings[playlistSetting];
        }

        // set boolean settings
        for ( let key in playerBoolSettings ) {
            if ( playerBoolSettings.hasOwnProperty( key ) ) {
                playerSettings[key] = playerBoolSettings[key] === 'yes';
            }
        }

        // Change to string from array
        mejs.i18n.en['mejs.time-skip-back'] = 'Skip back %1 seconds';
        mejs.i18n.en['mejs.time-jump-forward'] = 'Jump forward %1 seconds';

        if ( isPlaylist ) {
            this.createPlaylist( widgetId, features, playerSettings )
        } else {
            this.createAudioPlayer( widgetId, features, playerSettings );
        }

        // Return to default value
        mejs.i18n.en['mejs.time-skip-back'] = ['Skip back 1 second', 'Skip back %1 seconds'];
        mejs.i18n.en['mejs.time-jump-forward'] = ['Jump forward 1 second', 'Jump forward %1 seconds'];


    },

    createPlaylist: function ( widgetId, features, playerSettings ) {
        const oldBuildFeatures = MediaElementPlayer.prototype.buildfeatures;
        const $mediaPlayer = document.querySelector( `#mdp-audier-elementor-player-${widgetId}` );

        features.push( 'playlist' );

        // override build features in mediaelement-migrate
        MediaElementPlayer.prototype.buildfeatures = function ( player, controls, layers, media ) {
            let t = this;
            player.container = document.querySelector( `#${player.id}.mejs-container` );

            for ( let i = 0, total = t.options.features.length; i < total; i++ ) {
                let feature = t.options.features[i];
                if ( t['build' + feature] ) {
                    try {
                        t['build' + feature]( player, controls, layers, media );
                    } catch ( e ) {
                        console.error( 'error building ' + feature, e );
                    }
                }
            }
        };


        if ( $mediaPlayer ) { new MediaElementPlayer( $mediaPlayer, playerSettings ); }

        // return to default
        MediaElementPlayer.prototype.buildfeatures = oldBuildFeatures;
    },

    createAudioPlayer: function ( widgetId, features, playerSettings ) {

        new MediaElementPlayer( `mdp-audier-elementor-player-${widgetId}`, playerSettings );
    }
}

/**
 * Init for Front-End
 * @param callback
 */
jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-audier-elementor' ) ) {
            mdpAudierElementor.audierElementor.call( mdpAudierElementor, $scope[0].dataset.id );
        }
    } );

} );