<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Audier for Elementor
 * Customizable audio player for Elementor editor
 * Exclusively on https://1.envato.market/audier-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.2
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Cherviakov Vlad (vladchervjakov@gmail.com), Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\AudierElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Repeater;
use Elementor\Utils;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\AudierElementor\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Audier - Custom Elementor Widget.
 **/
class audier_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
         'mdp-audier-elementor-admin',
            UnityPlugin::get_url() . 'src/Merkulove/Unity/assets/css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );
        wp_register_style(
         'mdp-audier-elementor',
            UnityPlugin::get_url() . 'css/audier-elementor' . UnityPlugin::get_suffix() . '.css',
                [ 'mediaelement' ],
                UnityPlugin::get_version()
        );

        wp_register_style(
         'mdp-audier-elementor-playlist',
            UnityPlugin::get_url() . 'css/lib/playlist' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );

        wp_register_style(
         'mdp-audier-elementor-skipback',
            UnityPlugin::get_url() . 'css/lib/skip-back' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );

        wp_register_style(
         'mdp-audier-elementor-jumpforward',
            UnityPlugin::get_url() . 'css/lib/jump-forward' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );

        wp_register_script(
            'mdp-audier-elementor-playlist',
            UnityPlugin::get_url() . 'js/lib/playlist' . UnityPlugin::get_suffix() . '.js',
            [],
            UnityPlugin::get_version(),
            true
        );

        wp_register_script(
            'mdp-audier-elementor-skipback',
            UnityPlugin::get_url() . 'js/lib/skip-back' . UnityPlugin::get_suffix() . '.js',
            [],
            UnityPlugin::get_version(),
            true
        );

        wp_register_script(
            'mdp-audier-elementor-jumpforward',
            UnityPlugin::get_url() . 'js/lib/jump-forward' . UnityPlugin::get_suffix() . '.js',
            [],
            UnityPlugin::get_version(),
            true
        );

	    wp_register_script(
                'mdp-audier-elementor',
                   UnityPlugin::get_url() . 'js/audier-elementor' . UnityPlugin::get_suffix() . '.js',
                       [ 'jquery', 'mediaelement' ],
                       UnityPlugin::get_version(),
              true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-audier-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Audier', 'audier-elementor' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-audier-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'general' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Audier' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [
            'mdp-audier-elementor',
            'mdp-audier-elementor-admin',
            'mdp-audier-elementor-skipback',
            'mdp-audier-elementor-playlist',
            'mdp-audier-elementor-jumpforward'
        ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [
            'mdp-audier-elementor',
            'mdp-audier-elementor-playlist',
            'mdp-audier-elementor-skipback',
            'mdp-audier-elementor-jumpforward'
        ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_content_general();

        /** Content -> Playlist Content Section. */
        $this->section_content_playlist();

        /** Content -> Audio Content Section. */
        $this->section_content_audio();

        /** Content -> Controls Content Section. */
        $this->section_content_controls();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Controls Box. */
        $this->section_style_controls_box();

        /** Style -> Section Style Pause Button. */
        $this->create_button_styles( 'section_style_pause_button', 'Pause button', 'mejs-pause' );

        /** Style -> Section Style Play Button. */
        $this->create_button_styles( 'section_style_play_button', 'Play button', 'mejs-play' );

        /** Style -> Section Style Mute Button. */
        $this->create_button_styles( 'section_style_mute_button', 'Mute button', 'mejs-mute' );

        /** Style -> Section Style Unmute Button. */
        $this->create_button_styles( 'section_style_unmute_button', 'Unmute button', 'mejs-unmute' );

        /** Style -> Section Style Previous Button. */
        $this->create_button_styles( 'section_style_previous_button', 'Previous button', 'mejs-prev-button' );

        /** Style -> Section Style Next Button. */
        $this->create_button_styles( 'section_style_next_button', 'Next button', 'mejs-next-button' );

        /** Style -> Section Style Loop Button. */
        $this->create_button_styles( 'section_style_loop_button', 'Loop button', 'mejs-loop-button' );

        /** Style -> Section Style Shuffle Button. */
        $this->create_button_styles( 'section_style_shuffle_button', 'Shuffle button', 'mejs-shuffle-button' );

        /** Style -> Section Style Skip Back Button. */
        $this->create_button_styles(
        'section_style_skip_back_button',
            'Skip back button',
          'mejs-skip-back-button',
      true
        );

        /** Style -> Section Style Jump Forward Button. */
        $this->create_button_styles(
        'section_style_jump_forward_button',
            'Jump forward button',
          'mejs-jump-forward-button',
      true
        );

        /** Style -> Section Style Current Time Block. */
        $this->create_button_styles(
         'section_style_current_time',
            'Current time',
          'mejs-currenttime-container',
      true,
 true
        );

        /** Style -> Section Style Duration Time Block. */
        $this->create_button_styles(
         'section_style_duration_time',
            'Duration',
          'mejs-duration-container',
      true,
 true
        );

        /** Style -> Section Style Progress Bar. */
        $this->section_style_progressbar();

        /** Style -> Section Style Horizontal Volume Bar. */
        $this->section_style_horizontal_volume_bar();

        /** Style -> Section Style Vertical Volume Bar. */
        $this->section_style_vertical_volume_bar();

        /** Style -> Section Style Playlist Box. */
        $this->section_style_playlist_box();

        /** Style -> Section Style Playlist Track. */
        $this->section_style_playlist_track();

        /** Style -> Section Style Playlist Track Image. */
        $this->section_style_playlist_track_image();

        /** Style -> Section Style Playlist Track Title. */
        $this->create_playlist_text_styles(
        'section_style_playlist_track_title',
            'Track title',
          'mejs-playlist-selector-label'
        );

        /** Style -> Section Style Playlist Track Description. */
        $this->create_playlist_text_styles(
        'section_style_playlist_track_description',
            'Track description',
          'mejs-playlist-item-description'
        );


        /** Style -> Section Style Playlist Current Track Title. */
        $this->create_playlist_text_styles(
            'section_style_playlist_current_track_title',
            'Current track title',
          'mejs-playlist-current-title'
        );

        /** Style -> Section Style Playlist Current Track Message. */
        $this->create_playlist_text_styles(
         'section_style_playlist_current_track_message',
            'Current track message',
          'mejs-playlist-current p:first-of-type'
        );

        /** Style -> Section Style Playlist Current Track Image. */
        $this->section_style_playlist_current_track_image();

        /** Style -> Section Style Playlist Current Track. */
        $this->section_style_playlist_current_track();

    }

    /**
     * Add widget controls: Content -> General Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_general() {

        $this->start_controls_section( 'section_content_general', [
            'label' => esc_html__( 'General', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'audio_source',
            [
                'label'     => esc_html__( 'Source', 'audier-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'self-hosted'    => esc_html__( 'Self hosted', 'audier-elementor' ),
                    'external'    => esc_html__( 'External', 'audier-elementor' ),
                    'playlist'    => esc_html__( 'Playlist', 'audier-elementor' ),
                ],
                'default'   => 'self-hosted',
                'separator' => 'none',
            ]
        );

        $this->add_control(
            'external_source_url',
            [
                'label' => esc_html__( 'Source', 'audier-elementor' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'audier-elementor' ),
                'options' => [ 'url', 'is_external', 'nofollow' ],
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
                'dynamic' => [
                    'active' => true,
                ],
                'label_block' => true,
                'condition' => [
                    'audio_source' => 'external'
                ]
            ]
        );

        $this->add_control(
            'self_hosted_url',
            [
                'label' => esc_html__( 'Audio', 'audier-elementor' ),
                'type' => Controls_Manager::MEDIA,
                'media_types' => ['audio'],
                'condition' => [
                    'audio_source' => 'self-hosted'
                ],
                'description' => esc_html__( 'Support MP3 audio format', 'audier-elementor' )
            ]
        );

        $this->add_control(
            'player_alignment',
            [
                'label' => esc_html__( 'Player alignment', 'audier-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-audier-elementor-audio-wrapper' => 'justify-content: {{VALUE}};',
                ],
                'default' => 'center',
                'toggle' => true,
                'condition' => [
                    'audio_source!' => 'playlist'
                ]
            ]
        );

        $this->add_control(
            'player_width',
            [
                'label' => esc_html__( 'Player width', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'size' => 100,
                    'unit' => '%'
                ],
                'condition' => [
                    'audio_source!' => 'playlist'
                ]
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Playlist Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_playlist() {

        $this->start_controls_section('section_content_playlist', [
            'label' => esc_html__('Playlist', 'audier-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'audio_source' => 'playlist'
            ]
        ] );

        $this->add_responsive_control(
            'playlist_height',
            [
                'label' => esc_html__( 'Playlist height', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-container.mejs-audio' => 'min-height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'player_playlist_track_content_alignment',
            [
                'label' => esc_html__( 'Playlist track content alignment', 'audier-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-content label' => 'text-align: {{VALUE}};',
                    '{{WRAPPER}} .mejs-playlist-item-description' => 'text-align: {{VALUE}};'
                ],
                'default' => 'left',
                'toggle' => true
            ]
        );

        $this->add_responsive_control(
            'player_playlist_track_image_alignment',
            [
                'label' => esc_html__( 'Playlist track image alignment', 'audier-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'audier-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail' => 'text-align: {{VALUE}};',
                ],
                'default' => 'left',
                'toggle' => true
            ]
        );


        $this->add_control(
            'settings_current_message',
            [
                'label' => esc_html__( 'Current media message', 'audier-elementor' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your text here', 'audier-elementor' ),
                'dynamic' => [
                    'active' => true,
                ],
                'description' => esc_html__(
                'Prepended to the title of media (i.e., Now playing)',
             'audier-elementor'
                )
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'playlist_audio_source',
            [
                'label'     => esc_html__( 'Source', 'audier-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'self-hosted'    => esc_html__( 'Self hosted', 'audier-elementor' ),
                    'external'    => esc_html__( 'External', 'audier-elementor' ),
                ],
                'default'   => 'self-hosted',
                'separator' => 'none',
            ]
        );

        $repeater->add_control(
            'playlist_self_hosted_url',
            [
                'label' => esc_html__( 'Audio', 'audier-elementor' ),
                'type' => Controls_Manager::MEDIA,
                'media_types' => ['audio'],
                'condition' => [
                    'playlist_audio_source' => 'self-hosted'
                ],
                'description' => esc_html__( 'Support MP3 audio format', 'audier-elementor' )
            ]
        );

        $repeater->add_control(
            'playlist_audio_source_url',
            [
                'label' => esc_html__( 'Source', 'audier-elementor' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'audier-elementor' ),
                'options' => [ 'url' ],
                'default' => [ 'url' => '' ],
                'label_block' => true,
                'dynamic' => [
                    'active' => true,
                ],
                'condition' => [
                    'playlist_audio_source' => 'external'
                ]
            ]
        );

        $repeater->add_control(
            'playlist_audio_title',
            [
                'label' => esc_html__( 'Audio title', 'audier-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Playlist audio' ),
                'dynamic' => [
                    'active' => true,
                ],
                'placeholder' => esc_html__( 'Type your text here', 'audier-elementor' ),
            ]
        );

        $repeater->add_control(
            'playlist_audio_description',
            [
                'label' => esc_html__( 'Audio description', 'audier-elementor' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'dynamic' => [
                    'active' => true,
                ],
                'placeholder' => esc_html__( 'Type your description here', 'audier-elementor' ),
            ]
        );

        $repeater->add_control(
            'playlist_audio_thumbnail',
            [
                'label' => esc_html__( 'Audio thumbnail', 'audier-elementor' ),
                'type' => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        $this->add_control(
            'playlist_audios',
            [
                'label' => esc_html__( 'Playlist', 'audier-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'title_field' => '{{{ playlist_audio_title }}}',
            ]
        );


        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Audio Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_audio() {

        $this->start_controls_section( 'section_content_audio', [
            'label' => esc_html__('Audio', 'audier-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'player_muted',
            [
                'label' => esc_html__( 'Muted', 'audier-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'enable_loop',
            [
                'label' => esc_html__( 'Loop', 'audier-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                    'audio_source!' => 'playlist'
                ]
            ]
        );

        $this->add_control(
            'enable_pause_other_players',
            [
                'label' => esc_html__( 'Pause other players', 'audier-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Creates player control setting.
     *
     * @param $id
     * @param $repeater
     * @param $title
     * @param $condition
     * @return mixed $repeater
     * @since 1.0.0
     * @access private
     *
     */
    private function create_control_text_setting( $id, $repeater, $title, $condition ) {
        return $repeater->add_control(
               'settings_' . $id,
               [
                    'label' => esc_html__( $title, 'audier-elementor' ),
                    'type' => Controls_Manager::TEXT,
                    'placeholder' => esc_html__( 'Type your text here', 'audier-elementor' ),
                   'dynamic' => [
                       'active' => true,
                   ],
                    'condition' => [
                        'controls_player_control' => $condition
                ]
            ]
        );
    }

    /**
     * Add widget controls: Content -> Controls Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_controls() {

        $this->start_controls_section( 'section_content_controls', [
            'label' => esc_html__('Controls', 'audier-elementor'),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $repeater = new Repeater();

        $repeater->add_control(
            'controls_player_control',
            [
                'label'     => esc_html__( 'Player control', 'audier-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'playpause'    => esc_html__( 'Play button', 'audier-elementor' ),
                    'progress'    => esc_html__( 'Progress bar', 'audier-elementor' ),
                    'duration'    => esc_html__( 'Duration', 'audier-elementor' ),
                    'current'     => esc_html__( 'Current time', 'audier-elementor' ),
                    'volume'     => esc_html__( 'Volume', 'audier-elementor' ),
                    'nexttrack'  => esc_html__( 'Next track', 'audier-elementor' ),
                    'prevtrack'  => esc_html__( 'Previous track', 'audier-elementor' ),
                    'skipback'  => esc_html__( 'Skip back', 'audier-elementor' ),
                    'jumpforward' => esc_html__( 'Jump forward', 'audier-elementor' ),
                    'loop' => esc_html__( 'Loop', 'audier-elementor' ),
                    'shuffle' => esc_html__( 'Shuffle', 'audier-elementor' ),
                ],
                'default'   => 'playpause',
                'separator' => 'none',
            ]
        );

        $this->create_control_text_setting( 'play_text', $repeater, 'Play button text', 'playpause' );

        $this->create_control_text_setting( 'pause_text', $repeater, 'Pause button text', 'playpause' );

        $this->create_control_text_setting( 'mute_text', $repeater, 'Mute button text', 'volume' );

        $this->create_control_text_setting( 'unmute_text', $repeater, 'Unmute button text', 'volume' );

        $this->create_control_text_setting( 'prev_text', $repeater, 'Previous button text', 'prevtrack' );

        $this->create_control_text_setting( 'next_text', $repeater, 'Next button text', 'nexttrack' );

        $this->create_control_text_setting( 'loop_text', $repeater, 'Loop button text', 'loop' );

        $this->create_control_text_setting( 'shuffle_text', $repeater, 'Shuffle button text', 'shuffle' );


        $repeater->add_control(
            'settings_audio_volume',
            [
                'label'     => esc_html__( 'Volume bar layout', 'audier-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'vertical'    => esc_html__( 'Vertical', 'audier-elementor' ),
                    'horizontal'    => esc_html__( 'Horizontal', 'audier-elementor' )
                ],
                'default'   => 'horizontal',
                'separator' => 'none',
                'condition' => [
                    'controls_player_control' => 'volume',
                ]
            ]
        );

        $repeater->add_control(
            'settings_skip_back_interval',
            [
                'label' => esc_html__( 'Skip back interval', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 30
                ],
                'condition' => [
                    'controls_player_control' => 'skipback'
                ]
            ]
        );

        $repeater->add_control(
            'settings_jump_forward_interval',
            [
                'label' => esc_html__( 'Jump forward interval', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 30
                ],
                'condition' => [
                    'controls_player_control' => 'jumpforward'
                ]
            ]
        );


        $this->add_control(
            'controls_player_controls',
            [
                'label' => esc_html__( 'Player controls', 'audier-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'controls_player_control' => esc_html__( 'playpause' )
                    ],
                    [
                        'controls_player_control' => esc_html__( 'current' )
                    ],
                    [
                        'controls_player_control' => esc_html__( 'progress' )
                    ],
                    [
                        'controls_player_control' => esc_html__( 'duration' )
                    ],
                    [
                        'controls_player_control' => esc_html__( 'volume' )
                    ],
                ],
                'title_field' => 'Control - {{{ controls_player_control }}}',
            ]
        );


        $this->add_control(
            'enable_enable_keyboard',
            [
                'label' => esc_html__( 'Keyboard support', 'audier-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'enable_hide_volume_on_touch_devices',
            [
                'label' => esc_html__( 'Hide volume on touch devices', 'audier-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'settings_start_volume',
            [
                'label' => esc_html__( 'Initial volume', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1,
                        'step' => 0.1,
                    ]
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0.8
                ]
            ]
        );

        $this->end_controls_section();
    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_additional_colors' => array_key_exists( 'include_additional_colors', $opts ) ?
                $opts['include_additional_colors'] : false,
            'include_hover_additional_colors' => array_key_exists( 'include_hover_additional_colors', $opts ) ?
                $opts['include_hover_additional_colors'] : false,
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_border' => array_key_exists( 'include_border', $opts ) ?
                $opts['include_border'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_bg'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'additional_background' => array_key_exists( 'additional_background', $opts ) ?
                $opts['additional_background'] : false,
            'typography_name' => array_key_exists( 'typography_name', $opts ) ?
                $opts['typography_name'] : 'Typography',
            'additional_background_name' => array_key_exists( 'additional_background_name', $opts ) ?
                $opts['additional_background_name'] : '',
            'additional_background_class' => array_key_exists( 'additional_background_class', $opts ) ?
                $opts['additional_background_class'] : '',
            'additional_border_radius_class' => array_key_exists( 'additional_border_radius_class', $opts ) ?
                $opts['additional_border_radius_class'] : '',
            'additional_border_radius_hover_class' => array_key_exists( 'additional_border_radius_hover_class', $opts ) ?
                $opts['additional_border_radius_hover_class'] : '',
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'FOCUS',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'additional_bg_class' => array_key_exists( 'additional_bg_class', $opts ) ?
                $opts['additional_bg_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_class' => array_key_exists( 'additional_color_hover_class', $opts ) ?
                $opts['additional_color_hover_class'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3,
            'additional_colors' => array_key_exists( 'additional_colors', $opts ) ?
                $opts['additional_colors'] : '',
            'hover_additional_colors' => array_key_exists( 'hover_additional_colors', $opts ) ?
                $opts['hover_additional_colors'] : [],
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__( $style_opts['typography_name'], 'audier-elementor' ),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'audier-elementor' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'audier-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'audier-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_additional_colors'] && is_array( $style_opts['additional_colors'] ) ) {
            foreach ( $style_opts['additional_colors'] as $additional_color ) {
                $this->add_control(
                    $section_id . '_' . $additional_color['name'] . '_normal_text_color',
                    [
                        'label' => esc_html__( $additional_color['name'], 'audier-elementor' ),
                        'type' => Controls_Manager::COLOR,
                        'selectors' => [
                            "{{WRAPPER}} .".$additional_color['class'] => $additional_color['value'],
                        ],
                    ]
                );
            }
        }

        if ( $style_opts['include_bg'] ) {
            $additional_bg_class = '';
            if ( !empty( $style_opts['additional_bg_class'] ) ) {
                $additional_bg_class = ', {{WRAPPER}} .'.$style_opts['additional_bg_class'];
            }

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'audier-elementor'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'] . $additional_bg_class,
                ]
            );

        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'audier-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'] => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_normal',
                    'label' => esc_html__('Border Type', 'audier-elementor'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'],
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'audier-elementor' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'audier-elementor' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'audier-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'audier-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['include_hover_additional_colors'] && is_array( $style_opts['hover_additional_colors'] ) ) {
            foreach ( $style_opts['hover_additional_colors'] as $additional_color ) {
                $this->add_control(
                    $section_id . '_' . $additional_color['name'] . '_hover_text_color',
                    [
                        'label' => esc_html__( $additional_color['name'], 'audier-elementor' ),
                        'type' => Controls_Manager::COLOR,
                        'selectors' => [
                            "{{WRAPPER}} .".$additional_color['class'] => $additional_color['value'],
                        ],
                    ]
                );
            }
        }

        if ( $style_opts['include_bg'] ) {
            $additional_bg_class = '';
            if ( !empty( $style_opts['additional_bg_class'] ) ) {
                $additional_bg_class = ', {{WRAPPER}} .'.$style_opts['additional_bg_class'] . ':hover';
            }

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'audier-elementor'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover" . $additional_bg_class,
                ]
            );
        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'audier-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'].':hover' => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }


        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'audier-elementor' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_hover',
                    'label' => esc_html__('Border Type', 'audier-elementor'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'] . ":hover",
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_hover_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'audier-elementor' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab(
                $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'audier-elementor' )]
            );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'audier-elementor '),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'audier-elementor' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'audier-elementor' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            if ( $style_opts['include_border'] ) {

                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => $section_id . '_border_active',
                        'label' => esc_html__('Border Type', 'audier-elementor'),
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );

            }

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'audier-elementor' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'audier-elementor' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Create button style settings.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_button_styles( $section_id, $name, $selector, $include_color = false, $include_typography = false ) {
        $this->start_controls_section( $section_id, [
            'label' => esc_html__( $name, 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );


        $this->add_responsive_control(
            $section_id . '_margin',
            [
                'label' => esc_html__( 'Margin', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} ." . $selector => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $typography_tabs_controls_options = [
            'html_class' => $selector,
            'include_typography' => false,
            'include_color' => false
        ];

        if ( $include_color ) {
            $typography_tabs_controls_options['include_color'] = true;
            $typography_tabs_controls_options['color_class'] = $selector . ' button, {{WRAPPER}} .' . $selector . ' span';
            $typography_tabs_controls_options['color_hover_class'] = $selector . ':hover button, {{WRAPPER}} .' . $selector . ':hover span';
        }

        if ( $include_typography ) {
            $typography_tabs_controls_options['include_typography'] = true;
            $typography_tabs_controls_options['typography_class'] = $selector . ', {{WRAPPER}} .' . $selector . ' span';
        }

        $this->generate_typography_tabs_controls( $section_id, $typography_tabs_controls_options );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Progressbar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_progressbar() {

        $this->start_controls_section( 'section_style_progressbar', [
            'label' => esc_html__( 'Progress bar', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );


        $this->add_responsive_control(
            'section_style_progressbar_margin',
            [
                'label' => esc_html__( 'Margin', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mejs-time-slider" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            'progressbar_width',
            [
                'label' => esc_html__( 'Progress bar width', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-time-total' => 'width: {{SIZE}}{{UNIT}};'
                ]
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_progressbar', [
            'html_class' => 'mejs-time-slider',
            'include_typography' => false,
            'include_color' => false,
            'include_additional_colors' => true,
            'include_hover_additional_colors' => true,
            'include_bg' => false,
            'additional_transition_selector' => '{{WRAPPER}} .mejs-time-current, {{WRAPPER}} .mejs-time-total',
            'additional_colors' => [
                [
                    'name' => 'Time loaded bar',
                    'class' => 'mejs-time-loaded',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time total bar',
                    'class' => 'mejs-time-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time current bar',
                    'class' => 'mejs-time-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ],
            'hover_additional_colors' => [
                [
                    'name' => 'Time loaded bar',
                    'class' => 'mejs-time-rail:hover .mejs-time-loaded',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time total bar',
                    'class' => 'mejs-time-rail:hover .mejs-time-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Time current bar',
                    'class' => 'mejs-time-rail:hover .mejs-time-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ]
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Horizontal Volume Bar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_horizontal_volume_bar() {

        $this->start_controls_section( 'section_style_horizontal_volume_bar', [
            'label' => esc_html__( 'Horizontal volume bar', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_horizontal_volume_bar_margin',
            [
                'label' => esc_html__( 'Margin', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mejs-horizontal-volume-total" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_horizontal_volume_bar', [
            'html_class' => 'mejs-horizontal-volume-total',
            'include_typography' => false,
            'include_color' => false,
            'include_additional_colors' => true,
            'include_hover_additional_colors' => true,
            'include_bg' => false,
            'additional_transition_selector' => '{{WRAPPER}} .mejs-horizontal-volume-current, {{WRAPPER}} .mejs-horizontal-volume-total',
            'additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-horizontal-volume-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-horizontal-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ],
            'hover_additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-horizontal-volume-total:hover',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-horizontal-volume-total:hover .mejs-horizontal-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ]
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Vertical Volume Bar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_vertical_volume_bar() {

        $this->start_controls_section( 'section_style_vertical_volume_bar', [
            'label' => esc_html__( 'Vertical volume bar', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_vertical_volume_bar_margin',
            [
                'label' => esc_html__( 'Margin', 'audier-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mejs-volume-slider" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_vertical_volume_bar', [
            'html_class' => 'mejs-volume-slider',
            'include_typography' => false,
            'include_color' => false,
            'include_additional_colors' => true,
            'include_hover_additional_colors' => true,
            'additional_transition_selector' => '{{WRAPPER}} .mejs-volume-current, {{WRAPPER}} .mejs-volume-total',
            'additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-volume-total',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ],
            'hover_additional_colors' => [
                [
                    'name' => 'Total volume bar',
                    'class' => 'mejs-volume-total:hover',
                    'value' => 'background: {{VALUE}} !important;'
                ],
                [
                    'name' => 'Current volume bar',
                    'class' => 'mejs-volume-total:hover .mejs-volume-current',
                    'value' => 'background: {{VALUE}} !important;'
                ],
            ]
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Controls Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_controls_box() {

        $this->start_controls_section( 'section_style_controls_box', [
            'label' => esc_html__( 'Controls box', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
        ] );

        $this->add_responsive_control(
            'section_style_controls_box_height',
            [
                'label' => esc_html__( 'Height', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-controls' => 'height: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-audier-elementor-not-playlist .mejs-container' => 'height: {{SIZE}}{{UNIT}} !important;'
                ]
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_controls_box', [
            'html_class' => 'mejs-controls',
            'include_typography' => false,
            'include_color' => false,
            'additional_border_radius_class' => '{{WRAPPER}} .mdp-audier-elementor-not-playlist .mejs-container ',
            'additional_border_radius_hover_class' => '{{WRAPPER}} .mdp-audier-elementor-not-playlist:hover .mejs-container'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Playlist Box.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_box() {

        $this->start_controls_section( 'section_style_playlist_box', [
            'label' => esc_html__('Playlist box', 'audier-elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'audio_source' => 'playlist'
            ]
        ] );

        $this->generate_typography_tabs_controls( 'section_style_playlist_box', [
            'html_class' => 'mejs-playlist-layer',
            'include_typography' => false,
            'include_color' => false,
            'additional_border_radius_class' => '{{WRAPPER}} .mejs-container ',
            'additional_border_radius_hover_class' => '{{WRAPPER}} .mejs-container:hover',
            'additional_bg_class' => 'mejs-container'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Playlist Track.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_track() {

        $this->start_controls_section( 'section_style_playlist_track', [
            'label' => esc_html__( 'Playlist track', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'audio_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_playlist_track',
            'mejs-playlist-selector-list-item'
        );

        $this->generate_typography_tabs_controls( 'section_style_playlist_track', [
            'html_class' => 'mejs-playlist-selector-list-item',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Playlist Current Track.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_current_track() {

        $this->start_controls_section( 'section_style_playlist_current_track', [
            'label' => esc_html__( 'Playlist current track', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'audio_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_playlist_current_track',
            'mejs-playlist-current'
        );

        $this->generate_typography_tabs_controls( 'section_style_playlist_current_track', [
            'html_class' => 'mejs-playlist-current',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Creates playlist text styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_playlist_text_styles( $section_id, $name, $selector ) {

        $this->start_controls_section( $section_id, [
            'label' => esc_html__( $name, 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'audio_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            $section_id,
            $selector
        );

        $this->generate_typography_tabs_controls( $section_id, [
            'html_class' => $selector,
            'typography_class' => $selector,
            'color_class' => $selector,
            'color_hover_class' => 'mejs-playlist-item-inner:hover .' . $selector,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Playlist Track Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_track_image() {

        $this->start_controls_section( 'section_style_playlist_track_image', [
            'label' => esc_html__( 'Playlist track image', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'audio_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_playlist_track_image',
            'mejs-playlist-item-thumbnail img'
        );

        $this->add_control(
            'playlist_track_image_object_fit',
            [
                'label'     => esc_html__( 'Object fit', 'audier-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'fill'    => esc_html__( 'Fill', 'audier-elementor' ),
                    'contain'    => esc_html__( 'Contain', 'audier-elementor' ),
                    'cover'    => esc_html__( 'Cover', 'audier-elementor' ),
                    'scale-down'    => esc_html__( 'Scale down', 'audier-elementor' ),
                    'none'    => esc_html__( 'None', 'audier-elementor' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail img' => 'object-fit: {{VALUE}};'
                ],
                'default'   => 'fill',
                'separator' => 'none',
            ]
        );

        $this->add_responsive_control(
            'section_style_playlist_track_image_width',
            [
                'label' => esc_html__( 'Width', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail img' => 'width: {{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'section_style_playlist_track_image_height',
            [
                'label' => esc_html__( 'Height', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-item-thumbnail img' => 'height: {{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_playlist_track_image', [
            'html_class' => 'mejs-playlist-item-thumbnail img',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Playlist Current Track Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_playlist_current_track_image() {

        $this->start_controls_section( 'section_style_playlist_current_track_image', [
            'label' => esc_html__( 'Playlist current track image', 'audier-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'audio_source' => 'playlist'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_playlist_current_track_image',
            'mejs-playlist-current img'
        );

        $this->add_control(
            'playlist_current_track_image_object_fit',
            [
                'label'     => esc_html__( 'Object fit', 'audier-elementor' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'fill'    => esc_html__( 'Fill', 'audier-elementor' ),
                    'contain'    => esc_html__( 'Contain', 'audier-elementor' ),
                    'cover'    => esc_html__( 'Cover', 'audier-elementor' ),
                    'scale-down'    => esc_html__( 'Scale down', 'audier-elementor' ),
                    'none'    => esc_html__( 'None', 'audier-elementor' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-current img' => 'object-fit: {{VALUE}};'
                ],
                'default'   => 'fill',
                'separator' => 'none',
            ]
        );



        $this->add_control(
            'section_style_playlist_current_track_image_width',
            [
                'label' => esc_html__( 'Width', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-current img' => 'width: {{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->add_control(
            'section_style_playlist_current_track_image_height',
            [
                'label' => esc_html__( 'Height', 'audier-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mejs-playlist-current img' => 'height: {{SIZE}}{{UNIT}}'
                ]
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_playlist_current_track_image', [
            'html_class' => 'mejs-playlist-current img',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }



    /**
     * Create audio player.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_audio_player( $settings ) {
        $audio_url = $settings['audio_source'] === 'self-hosted' ?
            esc_url( $settings['self_hosted_url']['url'] ) :
            esc_url( $settings['external_source_url']['url'] );
        $is_editor = Plugin::instance()->editor->is_edit_mode();
        $admin_error_message = $is_editor ?
            esc_html__( 'You need to add source of your audio to display player!', 'audier-elementor' ) :
            '';

        return !empty( $audio_url ) ? sprintf(
            '<div class="mdp-audier-elementor-audio-wrapper">
                        <audio id="mdp-audier-elementor-player-%s" width="%s" %s>
                            <source src="%s" type="audio/mp3">
                        </audio>
                    </div>',
            $this->get_id(),
                    $settings['player_width']['size'] . $settings['player_width']['unit'],
                    $settings['player_muted'] === 'yes' ? 'muted' : '',
                    $audio_url

        ) : $admin_error_message;
    }

    /**
     * Create playlist videos.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_playlist_tracks( $playlist ) {
        $playlist_tracks = [];

        foreach ( $playlist as $playlist_item ) {
            $track_src = $playlist_item['playlist_audio_source'] === 'self-hosted' ?
                esc_url( $playlist_item['playlist_self_hosted_url']['url'] ) :
                esc_url( $playlist_item['playlist_audio_source_url']['url'] );
            $track_thumbnail = !empty( $playlist_item['playlist_audio_thumbnail']['url'] ) ?
               sprintf(
            'data-playlist-thumbnail="%s"',
                    esc_url( $playlist_item['playlist_audio_thumbnail']['url'] )
               ) : '';
            $track_title = !empty( $playlist_item['playlist_audio_title'] ) ?
                sprintf( 'title="%s"', esc_attr( $playlist_item['playlist_audio_title'] ) ) : '';
            $track_description = !empty( $playlist_item['playlist_audio_description'] ) ?
                sprintf(
                    'data-playlist-description="%s"',
                    esc_attr( $playlist_item['playlist_audio_description'] )
                ) : '';

            if ( !empty( $track_src ) && !empty( $track_title ) ) {
                $playlist_tracks[] = sprintf(
                    '<source src="%s"  type="audio/mp3" %s %s %s >',
                    $track_src,
                    $track_title,
                    $track_thumbnail,
                    $track_description
                );
            }
        }

        return count( $playlist_tracks ) >= 2 ? implode( ' ', $playlist_tracks ) : '';

    }

    /**
     * Create playlist.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_playlist( $settings ) {
        $playlist_tracks = $this->create_playlist_tracks( $settings['playlist_audios'] );
        $is_editor = Plugin::instance()->editor->is_edit_mode();
        $admin_error_message = $is_editor ?
            esc_html__( 'You need to add at least two audios with source and title!', 'audier-elementor' ) :
            '';

        return !empty( $playlist_tracks ) ? sprintf(
        '<audio 
               id="mdp-audier-elementor-player-%s" 
               class="mdp-audier-elementor-player" 
               width="100%%"
               %s >
               %s
        </audio>',
$this->get_id(),
        $settings['player_muted'] === 'yes' ? 'muted' : '',
        $playlist_tracks
        ) : $admin_error_message;
    }

    /**
     * Create audio player controls settings.
     *
     * @return array
     * @since 1.0.0
     * @access private
     *
     */
    private function get_player_controls( $settings ) {
        $settings_controls = $settings['controls_player_controls'];
        $controls = [];

        foreach ( $settings_controls as $settings_control ) {
            $controls[] = esc_attr( $settings_control['controls_player_control'] );
        }


        return array_values( array_unique( $controls ) );

    }

    /**
     * Converts string to camel case.
     *
     * @param $string
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function to_camel_case( $string ) {

        $str = str_replace( ' ', '', ucwords( str_replace( '_', ' ', $string ) ) );

        $str[0] = strtolower( $str[0] );

        return $str;
    }

    /**
     * Get boolean settings for player.
     *
     * @param $settings
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_player_boolean_settings( $settings ) {
        $player_settings = [];

        foreach ( $settings as $key => $setting ) {
            if ( strpos( $key, 'enable' ) !== false ) {
                $key_parts = explode( '_', $key, 2 );
                $player_settings[$this->to_camel_case( $key_parts[1] )] = $setting;
            }
        }

        return $player_settings;

    }

    /**
     * Get settings for player.
     *
     * @param $settings
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_player_settings( $settings ) {
        $player_settings = [];
        $controls_settings = $settings['controls_player_controls'];

        /** Set controls settings. */
        foreach ( $controls_settings as $controls_setting ) {
            foreach ( $controls_setting as $key => $controls_setting_item ) {
                if ( strpos( $key, 'settings' ) !== false ) {
                    $key_parts = explode( '_', $key, 2 );
                    if ( empty( $controls_setting_item ) ) { continue; }
                    $player_settings[$this->to_camel_case( $key_parts[1] )] = is_array( $controls_setting_item ) ?
                        $controls_setting_item['size'] : $controls_setting_item;

                }
            }
        }

        /** Set other settings. */
        foreach ( $settings as $key => $setting ) {
            if ( strpos( $key, 'settings' ) !== false ) {
                $key_parts = explode( '_', $key, 2 );
                if ( empty( $setting ) ) { continue; }
                $player_settings[$this->to_camel_case( $key_parts[1] )] = is_array( $setting ) ?
                    $setting['size'] : $setting;

            }
        }

        return $player_settings;

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        $player_controls = json_encode( $this->get_player_controls( $settings ) );
        $player_bool_settings = json_encode( $this->get_player_boolean_settings( $settings ) );
        $audio_source = esc_attr( $settings['audio_source'] );
        $player_settings = json_encode( $this->get_player_settings( $settings ) );

        $not_playlist_class = $audio_source !== 'playlist' ? 'mdp-audier-elementor-not-playlist' : '';

        echo sprintf(
            '<!-- Start Audier for Elementor WordPress Plugin -->
                    <div 
                        class="mdp-audier-elementor-box %s"
                        data-controls="%s"
                        data-player-bool-settings="%s"
                        data-player-settings="%s"
                        data-source="%s"
                    >%s</div>
                   <!-- End Audier for Elementor WordPress Plugin -->',
          $not_playlist_class,
                  esc_attr( $player_controls ),
                  esc_attr( $player_bool_settings ),
                  esc_attr( $player_settings ),
                  $audio_source,
                  $audio_source !== 'playlist' ?
                      $this->create_audio_player( $settings ) :
                      $this->create_playlist( $settings )
        );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/audier';

    }

}
